/*  XMMS - ALSA output plugin
 *    Copyright (C) 2001      Matthieu Sozeau
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "alsa.h"
#include <stdio.h>
static GtkWidget *configure_win = NULL;
static GtkWidget *mixer_usemaster_check, *buffer_size_spin, *buffer_pre_spin;
static GtkWidget *mixer_surround_check;
static GtkWidget *use_user_device_check, *user_device_entry;
static gint audio_device, mixer_device;
static my_index = 0;

typedef struct alsa_device_item
{
  gint numcard, numdevice;
  gchar label[128];
} alsa_device_item_t;

static alsa_device_item_t dev[5];


static void configure_win_ok_cb(GtkWidget * w, gpointer data)
{
  ConfigFile *cfgfile;

  alsa_cfg.use_user_device = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(use_user_device_check));
  alsa_cfg.user_device = gtk_editable_get_chars(GTK_EDITABLE(user_device_entry), 0, -1);
  alsa_cfg.audio_device = dev[audio_device].numdevice;
  alsa_cfg.audio_card = dev[audio_device].numcard;
  alsa_cfg.buffer_time = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(buffer_size_spin));
  alsa_cfg.use_master = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(mixer_usemaster_check));
  alsa_cfg.surround = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(mixer_surround_check));
  

  cfgfile = xmms_cfg_open_default_file();
  
  xmms_cfg_write_int(cfgfile, "ALSA", "audio_card", alsa_cfg.audio_card);
  xmms_cfg_write_int(cfgfile, "ALSA", "audio_device", alsa_cfg.audio_device);
  xmms_cfg_write_int(cfgfile, "ALSA", "buffer_time", alsa_cfg.buffer_time);
  xmms_cfg_write_boolean(cfgfile,"ALSA","use_master",alsa_cfg.use_master);
  xmms_cfg_write_boolean(cfgfile,"ALSA","surround",alsa_cfg.surround);
  xmms_cfg_write_boolean(cfgfile,"ALSA","use_user_device",alsa_cfg.use_user_device);
  xmms_cfg_write_string(cfgfile,"ALSA","user_device", alsa_cfg.user_device);
  xmms_cfg_write_default_file(cfgfile);
  xmms_cfg_free(cfgfile);
  
  gtk_widget_destroy(configure_win);
}

static void configure_win_audio_dev_cb(GtkWidget * widget, gint device)
{
  audio_device = device;
}

static void user_device_toggled(GtkToggleButton * widget, gpointer data)
{
  gboolean use_user_device = gtk_toggle_button_get_active(widget);
  gtk_widget_set_sensitive(GTK_WIDGET(data), !use_user_device);
  gtk_widget_set_sensitive(user_device_entry, use_user_device);
}

int contains(gchar *buf, int len, gchar *motif)
{
  int i, j;
  int lm;
  lm = strlen(motif);
  j = 0;
  for(i = 0; i < len; i++)
    {
      while(buf[i] == motif[j])
	{
	  i++;
	  j++;
	  if(motif[j] == '\0') return(1);
	}  
    }
  return(0);
}

void get_alsa_name(gchar *b, alsa_device_item_t *dev)
{
  int i = 0, cpt = 0;
  char tmp;
  int len;
  strncpy(&tmp, b + 1, 1);
  dev->numcard = (int) tmp - 48;
  strncpy(&tmp, b + 4, 1);
  dev->numdevice = (int) tmp - 48;
  
  do 
    {
      b = index(b, ':') + 1;
      cpt++;
    }while(cpt < 2);
  (char *) i = index(b, ':');
  len = ((int) i - (int) b);

  strncpy(dev->label, b + 1, len - 1);
  dev->label[len - 2] = '\0';

}



static void scan_devices(gchar * type, GtkWidget * option_menu, GtkSignalFunc sigfunc)
{
  GtkWidget *menu, *item;
  FILE *file;
  gchar buffer[256], *temp, *tmp2;
  gboolean found = FALSE;
  gint index = 0;

  menu = gtk_menu_new();
  
  if(!strcmp(type, "Audio devices:"))
    {
      my_index = 0;
      if ((file = fopen("/proc/asound/pcm", "r")))
	{
	  while(fgets(buffer, 255, file))
	    {
	      if(contains(buffer, strlen(buffer), "playback"))
		{
		  get_alsa_name(buffer, &dev[my_index]);
		  if(alsa_cfg.debug) printf("Label: %s Card: %i Device: %i\n", dev[my_index].label, dev[my_index].numcard, dev[my_index].numdevice);

		  item = gtk_menu_item_new_with_label(dev[my_index].label);
		  gtk_signal_connect(GTK_OBJECT(item), "activate", sigfunc, (gpointer) index++);
		  gtk_widget_show(item);
		  gtk_menu_append(GTK_MENU(menu), item);
		  my_index++;
		}
	    }
	  fclose(file);
	}
    }

  gtk_option_menu_set_menu(GTK_OPTION_MENU(option_menu), menu);  
}

void alsa_configure(void)
{
  GtkWidget *vbox, *notebook;
  GtkWidget *dev_vbox, *adevice_frame, *adevice_box, *adevice;
  GtkWidget *buffer_frame, *buffer_vbox, *buffer_table;
  GtkWidget *buffer_size_box, *buffer_size_label;
  GtkObject *buffer_size_adj, *buffer_pre_adj;
  GtkWidget *buffer_pre_box, *buffer_pre_label;
  
  GtkWidget *bbox, *ok, *cancel;
  GtkWidget *mixer_table, *mixer_frame;

  if (configure_win)
    {
      gdk_window_raise(configure_win->window);
      return;
    }
  
  configure_win = gtk_window_new(GTK_WINDOW_DIALOG);
  gtk_signal_connect(GTK_OBJECT(configure_win), "destroy", GTK_SIGNAL_FUNC(gtk_widget_destroyed), &configure_win);
  gtk_window_set_title(GTK_WINDOW(configure_win), _("ALSA Driver configuration"));
  gtk_window_set_policy(GTK_WINDOW(configure_win), FALSE, FALSE, FALSE);
  gtk_window_set_position(GTK_WINDOW(configure_win), GTK_WIN_POS_MOUSE);
  gtk_container_border_width(GTK_CONTAINER(configure_win), 10);
  
  vbox = gtk_vbox_new(FALSE, 10);
  gtk_container_add(GTK_CONTAINER(configure_win), vbox);
  
  notebook = gtk_notebook_new();
  gtk_box_pack_start(GTK_BOX(vbox), notebook, TRUE, TRUE, 0);
  
  dev_vbox = gtk_vbox_new(FALSE, 5);
  gtk_container_set_border_width(GTK_CONTAINER(dev_vbox), 5);
  
  adevice_frame = gtk_frame_new(_("Audio device:"));
  gtk_box_pack_start(GTK_BOX(dev_vbox), adevice_frame, FALSE, FALSE, 0);
  
  adevice_box = gtk_vbox_new(FALSE, 5);
  gtk_container_set_border_width(GTK_CONTAINER(adevice_box), 5);
  gtk_container_add(GTK_CONTAINER(adevice_frame), adevice_box);

  adevice = gtk_option_menu_new();
  gtk_box_pack_start(GTK_BOX(adevice_box), adevice, TRUE, TRUE, 0);
  scan_devices("Audio devices:", adevice, configure_win_audio_dev_cb);
  
  audio_device = alsa_cfg.audio_device;
  gtk_option_menu_set_history(GTK_OPTION_MENU(adevice), alsa_cfg.audio_device);

  use_user_device_check = gtk_check_button_new_with_label(_("User defined:"));
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(use_user_device_check), alsa_cfg.use_user_device);
  gtk_signal_connect(GTK_OBJECT(use_user_device_check), "toggled", user_device_toggled, adevice);
  gtk_box_pack_start(GTK_BOX(adevice_box), use_user_device_check, FALSE, FALSE, 0);

  user_device_entry = gtk_entry_new();
  gtk_entry_set_text(GTK_ENTRY(user_device_entry), alsa_cfg.user_device);
  gtk_box_pack_start(GTK_BOX(adevice_box), user_device_entry, TRUE, TRUE, 0);  
  
  gtk_notebook_append_page(GTK_NOTEBOOK(notebook), dev_vbox, gtk_label_new(_("Devices")));
  
  buffer_frame = gtk_frame_new(_("Buffering:"));
  gtk_container_set_border_width(GTK_CONTAINER(buffer_frame), 5);
  
  buffer_vbox = gtk_vbox_new(FALSE, 0);
  gtk_container_add(GTK_CONTAINER(buffer_frame), buffer_vbox);
  
  buffer_table = gtk_table_new(2, 1, TRUE);
  gtk_container_set_border_width(GTK_CONTAINER(buffer_table), 5);
  gtk_box_pack_start(GTK_BOX(buffer_vbox), buffer_table, FALSE, FALSE, 0);
  
  buffer_size_box = gtk_hbox_new(FALSE, 5);
  gtk_table_attach_defaults(GTK_TABLE(buffer_table), buffer_size_box, 0, 1, 0, 1);
  buffer_size_label = gtk_label_new(_("Buffer size (ms):"));
  gtk_box_pack_start(GTK_BOX(buffer_size_box), buffer_size_label, FALSE, FALSE, 0);

  buffer_size_adj = gtk_adjustment_new(alsa_cfg.buffer_time, 200, 1000000, 100, 100, 100);
  buffer_size_spin = gtk_spin_button_new(GTK_ADJUSTMENT(buffer_size_adj), 8, 0);
  gtk_widget_set_usize(buffer_size_spin, 60, -1);
  gtk_box_pack_start(GTK_BOX(buffer_size_box), buffer_size_spin, FALSE, FALSE, 0);
  
  gtk_notebook_append_page(GTK_NOTEBOOK(notebook), buffer_frame, gtk_label_new(_("Buffering")));
  
  mixer_frame = gtk_frame_new(_("Mixer Settings:"));
  gtk_container_set_border_width(GTK_CONTAINER(mixer_frame), 5);
  mixer_table = gtk_table_new(3, 2, TRUE);
  gtk_container_add(GTK_CONTAINER(mixer_frame), mixer_table);
  gtk_container_set_border_width(GTK_CONTAINER(mixer_table), 5);
  
  mixer_usemaster_check=gtk_check_button_new_with_label(_("Volume controls Master not PCM"));
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(mixer_usemaster_check),alsa_cfg.use_master);
  gtk_table_attach_defaults(GTK_TABLE(mixer_table), mixer_usemaster_check, 0, 1, 0, 1);
  
  mixer_surround_check=gtk_check_button_new_with_label(_("Enable Surround"));
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(mixer_surround_check),alsa_cfg.surround);
  gtk_table_attach_defaults(GTK_TABLE(mixer_table), mixer_surround_check, 0, 1, 1, 2);
  
  gtk_notebook_append_page(GTK_NOTEBOOK(notebook), mixer_frame, gtk_label_new(_("Mixer")));
 
  bbox = gtk_hbutton_box_new();
  gtk_button_box_set_layout(GTK_BUTTON_BOX(bbox), GTK_BUTTONBOX_END);
  gtk_button_box_set_spacing(GTK_BUTTON_BOX(bbox), 5);
  gtk_box_pack_start(GTK_BOX(vbox), bbox, FALSE, FALSE, 0);
  
  ok = gtk_button_new_with_label(_("Ok"));
  gtk_signal_connect(GTK_OBJECT(ok), "clicked", GTK_SIGNAL_FUNC(configure_win_ok_cb), NULL);
  GTK_WIDGET_SET_FLAGS(ok, GTK_CAN_DEFAULT);
  gtk_box_pack_start(GTK_BOX(bbox), ok, TRUE, TRUE, 0);
  gtk_widget_grab_default(ok);
  
  cancel = gtk_button_new_with_label(_("Cancel"));
  gtk_signal_connect_object(GTK_OBJECT(cancel), "clicked", GTK_SIGNAL_FUNC(gtk_widget_destroy), GTK_OBJECT(configure_win));
  GTK_WIDGET_SET_FLAGS(cancel, GTK_CAN_DEFAULT);
  gtk_box_pack_start(GTK_BOX(bbox), cancel, TRUE, TRUE, 0);
  
  gtk_widget_show_all(configure_win);
  
}
